# Copyright © CZ.NIC, z. s. p. o.
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from setuptools import setup

with open("README.md", "r") as readme:
    long_description = readme.read()


def read_requirements(filename="requirements.txt"):
    def valid_line(line):
        line = line.strip()
        return line and not any(line.startswith(p) for p in ("#", "-"))

    def extract_requirement(line):
        egg_eq = "#egg="
        if egg_eq in line:
            _, requirement = line.split(egg_eq, 1)
            return requirement
        return line

    with open(filename) as f:
        lines = f.readlines()
        return list(map(extract_requirement, filter(valid_line, lines)))


setup(
    name="fenix-checker",
    use_scm_version=True,
    packages=["fenix_checker"],
    setup_requires=["setuptools_scm"],
    description="A small server which checks if user's IP is in a trusted Fenix network.",
    author="Jiri Helebrant",
    author_email="jiri.helebrant@nic.cz",
    url="https://gitlab.labs.nic.cz/websites/fenix-checker",
    install_requires=read_requirements(),
    keywords=["fenix"],
    classifiers=[
        "Development Status :: 4 - Beta",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Operating System :: OS Independent"
    ],
    python_requires=">=3.6",
    long_description=long_description,
    long_description_content_type="text/markdown"
)



import json
from ipaddress import IPv4Address, IPv6Address
from sys import exit, stderr

import psycopg2
import yaml
from flask import Flask, Response, request

app = Flask(__name__)
application = app  # for uWSGI


def load_config(filename="config.yml"):
    try:
        conf_file = open(filename, "r")
    except FileNotFoundError:
        stderr.write(f"Config file ({filename}) not found.\n")
        exit(1)
    config = yaml.load(conf_file, Loader=yaml.SafeLoader)
    if "db" not in config:
        stderr.write("DB config not found in the configuration file.\n")
        exit(1)
    if set(["host", "port", "user", "password", "dbname"]) != set(config["db"]):
        stderr.write("Invalid DB config format.\n")
        exit(1)
    return config


def is_valid_ipv6_address(ip):
    try:
        addr = IPv6Address(ip)
    except ValueError:
        return False
    return addr.is_global


def is_valid_ipv4_address(ip):
    try:
        addr = IPv4Address(ip)
    except ValueError:
        return False
    return addr.is_global


def is_valid_ip_address(ip):
    return is_valid_ipv4_address(ip) or is_valid_ipv6_address(ip)


def json_response(content, status=200):
    return Response(json.dumps(content), status=status, headers={
        "Access-Control-Allow-Origin": "*",
        "Content-type": "application/json"
    })


@app.route("/", methods=["GET"])
def fenix():
    config = load_config()
    try:
        db_conn = psycopg2.connect(host=config["db"]["host"],
                                   port=config["db"]["port"],
                                   user=config["db"]["user"],
                                   password=config["db"]["password"],
                                   dbname=config["db"]["dbname"])

    except (psycopg2.OperationalError, psycopg2.DatabaseError):
        return json_response({"error": "Can't connect to DB"}, 400)

    if request.headers.getlist("X-Forwarded-For"):
        ip = request.headers.getlist("X-Forwarded-For")[0]
    else:
        ip = request.remote_addr
    if not is_valid_ip_address(ip):
        return json_response({"error": "Invalid IP"}, 400)
    db = db_conn.cursor()
    try:
        db.execute(f"SELECT 1 FROM networks WHERE inet '{ip}' << prefix")
    except (psycopg2.DataError, psycopg2.DatabaseError):
        return json_response({"error": "DB query error"}, 400)
    r = db.fetchall()
    db.close()
    if r == []:
        return json_response({"result": 0})
    elif int(r[0][0]) > 0:
        return json_response({"result": int(r[0][0])})
